def __validate_and_return_number(n):
    """RETURNS A FUNCTION which on execution raises a TypeError"""

    def raise_TypeError_func(n):
        def raise_TypeError():
            raise TypeError(f"{n} is not a number")

        return raise_TypeError

    def check_and_convert_string_func(s):
        """RETURNS A FUNCTION which on execution checks the string passed is a valid number or not"""

        def check_and_convert_string():
            if s.isdigit():
                return int(s)
            try:
                return float(s)
            except:
                raise_TypeError_func(s)()

        return check_and_convert_string

    return {
        "str": check_and_convert_string_func(n),
        "int": lambda: n,
        "float": lambda: n,
    }.get(type(n).__name__, raise_TypeError_func(n))()


def add(a, b):
    a = __validate_and_return_number(a)
    b = __validate_and_return_number(b)
    result = a + b
    if isinstance(result, float):
        return round(result, 16)
    return result


def sub(a, b):
    a = __validate_and_return_number(a)
    b = __validate_and_return_number(b)
    result = a - b
    if isinstance(result, float):
        return round(result, 16)
    return result


def mul(a, b):
    a = __validate_and_return_number(a)
    b = __validate_and_return_number(b)
    result = a * b
    if isinstance(result, float):
        return round(result, 16)
    return result


def div(a, b):
    a = __validate_and_return_number(a)
    b = __validate_and_return_number(b)
    result = a / b
    if isinstance(result, float):
        return round(result, 16)
    return result



from setuptools import setup, find_packages

BASE_DIR = "/home/manas/Codes/uploading_to_pypi/"

setup(
    name="mmmm_calculator",
    version="0.1.0",
    author="Manas Mishra",
    author_email="manas.m22@gmail.com",
    description=("Basic calculator to test uploading packages to pypi"),
    license="MIT",
    keywords="",
    url="https://github.com/manasm11/uploading_to_pypi",
    packages=find_packages(include=["mmmm_calculator", "mmmm_calculator.*"]),
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Other Audience",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
    ],
    install_requires=open(BASE_DIR + "requirements.txt").readlines(),
)
